@@ -6,6 +6,7 @@ __date__ ="$2011-09-04$"
 
 import dbus
 import dbus.glib
+import dbus.service
 import osso
 
 import sys
@@ -26,6 +27,12 @@ from flickcharm import FlickCharm
 import shutil
 import fnmatch
 
+import ExifTags
+from PIL import Image
+from PIL.ExifTags import TAGS
+
+import threading
+
 
 # Define some constants
 MCE_SIGNAL_NAME = 'sig_device_orientation_ind'
@@ -37,33 +44,68 @@ MCE_SERVICE = 'com.nokia.mce'
 MCE_REQUEST_PATH =  '/com/nokia/mce/request'
 MCE_REQUEST_IF = 'com.nokia.mce.request'
 MCE_DEVICE_ORIENTATION_GET = 'get_device_orientation'
+MCE_DEVICE_DISPSTATUS_GET = 'get_display_status'
+MCE_DEVICE_SCRLOCK_GET = 'get_tklock_mode'
+
+ENABLE_LED = 'req_led_pattern_activate'
+DISABLE_LED = 'req_led_pattern_deactivate'
+VIBRATE  = 'req_vibrator_pattern_activate'
+VIBRATE_PATTERN= 'PatternChatAndEmail'
+LED_PATTERN ='PatternCommunicationSMS'# 'PatternCommunicationIM'
 
 SH_DBUS_NAME  ="cn.l32606.maebo"
 SH_DBUS_IFACE ="cn.l32606.maebo"
 SH_DBUS_PATH  ="/cn/l32606/maebo"
 SH_SHARE_SIG  ="send_via_service"
 
+SESSION_DBUS_NAME  ="cn.l32606.maebo_session"
+SESSION_DBUS_IFACE ="cn.l32606.maebo_session"
+SESSION_DBUS_PATH  ="/cn/l32606/maebo_session"
+SESSION_SHARE_SIG  ="show_window"
+
+NOTIFY_DBUS_NAME  ="cn.l32606.qt_pynotify"
+NOTIFY_DBUS_IFACE ="cn.l32606.qt_pynotify"
+NOTIFY_DBUS_PATH  ="/cn/l32606/qt_pynotify"
+NOTIFY_SHARE_SIG_SHOW  ="show_notify"
+NOTIFY_SHARE_SIG_UPDATE  ="update_notify"
+NOTIFY_SHARE_SIG_QUIT  ="quit_app"
+
+_SM_AUTO_IDLE = 0
+_SM_AUTO_CONNECTING = 1
+_SM_AUTO_CONNECTED = 2
+
+
+
 class MaoboApp(QApplication):
     def __init__(self, parent=None):
-        self.logger = logging.getLogger('maebo')
+        print str(time.time()) + ": " + "MaoboApp __init__"
+        self.logger = logging.getLogger('maebo')        
         self.logger.info("__initing__")
+        
+    # Get the system bus
+        sysbus=dbus.Bus.get_system()
+
+    # Connect the "handler" to the orientation changed signal of mce
+        sysbus.add_signal_receiver(self.OrientationHandler, signal_name=MCE_SIGNAL_NAME, dbus_interface=MCE_SIGNAL_IFACE, path=MCE_SIGNAL_PATH)
+
+        self.osso_c = osso.Context(SH_DBUS_NAME,"0.0.1",False)
+        self.osso_rpc = osso.Rpc(self.osso_c)
+        self.osso_rpc.set_rpc_callback(SH_DBUS_NAME, SH_DBUS_PATH, SH_DBUS_IFACE, self.send_via_service, self.osso_c)
+
+        print str(time.time()) + ": " + "MaoboApp __init__ a" 
         super(MaoboApp, self).__init__(parent)
         self.charm = FlickCharm()
+
+        #print str(time.time()) + ": " + "MaoboApp __init__ b"   
         self.controller=Controller()
-        
-        #self.updater=Updater(self.show_notice)
-        #self.sender=Sender(self.show_notice)
+
+        print str(time.time()) + ": " + "MaoboApp __init__ c"   
         
         self.updatersina=UpdaterSina(self.show_notice)
         self.sendersina=SenderSina(self.show_notice)
         self.updaterqq=UpdaterQQ(self.show_notice)
         self.senderqq=SenderQQ(self.show_notice)
         
-        #self.connect(self.updater, SIGNAL("finished(bool)"), self.update_finished)
-        #self.connect(self.sender, SIGNAL("finished(bool)"), self.send_finished)
-        #self.connect(self.updater, SIGNAL("show_notice(QString,int)"), self.show_notice)
-        #self.connect(self.sender, SIGNAL("show_notice(QString,int)"), self.show_notice)
-        
         self.connect(self.updatersina, SIGNAL("finished(bool)"), self.update_finished)
         self.connect(self.sendersina, SIGNAL("finished(bool)"), self.send_finished)
         self.connect(self.updatersina, SIGNAL("show_notice(QString,int)"), self.show_notice)
@@ -75,7 +117,7 @@ class MaoboApp(QApplication):
         self.connect(self.updaterqq, SIGNAL("show_notice(QString,int)"), self.show_notice)
         self.connect(self.senderqq, SIGNAL("show_notice(QString,int)"), self.show_notice)
         self.connect(self.updaterqq, SIGNAL("cm_finished(bool)"), self.update_comments_finished)
-        
+
         self.currentStatus=None
         self.timelineType=None
         self.accounts=self.controller.get_all_accounts()
@@ -85,50 +127,225 @@ class MaoboApp(QApplication):
         self.currentAccount=None
 
         self.commentid_seed = 0
-        self.create_default_window()
         self.create_accounts_window()
-        #self.create_menus_window()
-        self.create_timeline_window()
-        self.create_status_window()
-        #self.newStatusWin=NewStatusWindow(self.menusWin, self.orientation)
-        self.newStatusWin=None
-        self.accountsManaForm=None
-        self.newAccountForm=None
 
-
-    # Get the system bus
-        sysbus=dbus.Bus.get_system()
-
-    # Connect the "handler" to the orientation changed signal of mce
-        sysbus.add_signal_receiver(self.OrientationHandler, signal_name=MCE_SIGNAL_NAME, dbus_interface=MCE_SIGNAL_IFACE, path=MCE_SIGNAL_PATH)
-
-        self.osso_c = osso.Context(SH_DBUS_NAME,"0.0.1",False)
-        self.osso_rpc = osso.Rpc(self.osso_c)
-        self.osso_rpc.set_rpc_callback(SH_DBUS_NAME, SH_DBUS_PATH, SH_DBUS_IFACE, self.send_via_service, self.osso_c)
-        
+        #print str(time.time()) + ": " + "MaoboApp __init__ 1"  
         
         self.sendersina_service=SenderSina(self.show_notice)
         self.senderqq_service=SenderQQ(self.show_notice)
 
-        self.clean_temp_images("/home/user/.sharing/outbox")
+        #self.clean_temp_images("/home/user/.sharing/outbox")
         
         Utils.g_orietation=self.get_current_orientation()
+        print Utils.g_orietation
 
+        print str(time.time()) + ": " + "MaoboApp __init__ 2"
         if len(self.accounts)==0:
+            self.create_default_window()
             self.defaultWin.show()
         else:
             self.accountsWin.show()
+        
+        self.create_timeline_window()
+        self.create_status_window()
+        self.newStatusWin=None
+        self.accountsManaForm=None
+        self.newAccountForm=None
+        self.defaultWin = None
+
+        self.unread_notifiers = {} #key = index of self.accounts
+        self.unread_updaters = {} #key = accountid in database
+        i = 0
+        for ac in self.accounts:
+            #print ac["id"], ac["website"], ac["nickname"]
+            #print ac["auto_update"]
             
+            if ac["website"] ==1:# and ac["auto_update"] == "True":
+                self.unread_updaters[str(ac["id"])]=UpdaterSina(self.show_notice, accountId=ac["id"])
+                self.unread_updaters[str(ac["id"])].OAuth(ac["username"],ac["password"])
+                self.connect(self.unread_updaters[str(ac["id"])], SIGNAL("unread_finished(int)"), self.get_unead_finished)
+            elif ac["website"] ==2:# and ac["auto_update"] == "True":    
+                self.unread_updaters[str(ac["id"])]=UpdaterQQ(self.show_notice, accountId=ac["id"])
+                self.unread_updaters[str(ac["id"])].OAuth(ac["username"],ac["password"])
+                self.connect(self.unread_updaters[str(ac["id"])], SIGNAL("unread_finished(int)"), self.get_unead_finished)
+
+            self.unread_notifiers[str(i)+":t"+str(Utils.TIMELINE_TYPE_MENTIONS)] = 0
+            self.unread_notifiers[str(i)+":t"+str(Utils.TIMELINE_TYPE_COMMENTS)] = 0
+            i = i + 1
+
+        print "first threading start"
+        self.sm_auto_update_unread = _SM_AUTO_IDLE
+        self.t = threading.Timer(5.0, self.cb_update_unread_counts)
+        self.t.start()
+        print str(time.time()) + ": " + "MaoboApp __init__ 3"
+        
+        self.connect(self, SIGNAL("lastWindowClosed()"), self.lastWindowClosed)
+        
+    def lastWindowClosed(self):
+        print "lastWindowClosed"        
+        '''TODO: need to check the updaters state, and terminate them'''
+
+
+    def change_current_timeline_view(self, account_idx, optype):
+        #print "change_current_timeline_view"
+
+        self.currentAccount = self.accounts[account_idx]
+        
+        if self.currentAccount["website"]==1:
+            self.updater=self.updatersina
+            self.sender=self.sendersina
+        else:
+            self.updater=self.updaterqq
+            self.sender=self.senderqq
+            
+        try:
+            self.updater.OAuth(self.currentAccount["username"],self.currentAccount["password"])
+            self.sender.OAuth(self.currentAccount["username"],self.currentAccount["password"])
+        except WeibopError,we:
+            self.show_notice(we)
+
+        self.timelineType=optype
+        self.timelineWin.currentMenuIdx = optype - 2
+        model = self.timelineWin.menuListView.model()
+        self.timelineWin.menuListView.setCurrentIndex(model.index(optype - 2, 0))
+        self.show_timeline_window()
+        
+        # clean unread counts
+        if self.timelineWin.menuModel.need_clear_unread_count(self.timelineType):
+            self.unread_updaters[str(self.currentAccount["id"])].initialize(self.currentAccount["id"], Utils.TIMELINE_TYPE_DEUNREAD, optype=self.timelineType) 
+            self.unread_updaters[str(self.currentAccount["id"])].start()
+        
+        self.update_timeline()        
+        
+        self.timelineWin.menuModel.clear_unread_count(self.timelineType) 
+        self.timelineWin.menuListView.hide()
+        self.timelineWin.menuListView.show()
+
+    def cb_update_unread_counts(self):
+        #print "cb_update_unread_counts"
+        
+        conn_status = self.osso_rpc.rpc_run("com.nokia.icd", "/com/nokia/icd", "com.nokia.icd", "get_state", wait_reply=True, use_system_bus=True)
+        #print "conn_status",conn_status
+
+        if conn_status == 0 and self.settings["auto_conn"] != "---None---":
+            #print "try to auto-connect internet - status: %s" % (self.sm_auto_update_unread)
+            self.sm_auto_update_unread = _SM_AUTO_CONNECTING
+
+            #dbus-send --type=method_call --system --dest=com.nokia.icd /com/nokia/icd com.nokia.icd.connect string:"d8a643a0-a0a6-4db2-9b87-759e6fd0f969" uint32:0
+            ret=self.osso_rpc.rpc_run("com.nokia.icd", "/com/nokia/icd", "com.nokia.icd", "connect", \
+                rpc_args=(self.settings["auto_conn"], 0), wait_reply=True, use_system_bus=True)
+
+            self.t = threading.Timer(15, self.cb_update_unread_counts)
+            self.t.start()
+            return
+        elif self.sm_auto_update_unread == _SM_AUTO_CONNECTING and conn_status == 1:
+            #print "auto-connect internet succeeded"
+            self.sm_auto_update_unread = _SM_AUTO_CONNECTED
+        
+        for acc in self.unread_updaters:
+            #print "cb_update_unread_counts: "+acc
+            if self.unread_updaters[acc].isRunning() != True:
+                #print "cb_update_unread_counts: update unread counts"
+                self.unread_updaters[acc].initialize(self.unread_updaters[acc].accountId,Utils.TIMELINE_TYPE_UNREAD)        
+                self.unread_updaters[acc].start()
+
+        if self.is_app_background_mode(): 
+            self.t = threading.Timer(60*float(self.settings["updateinterval2"]), self.cb_update_unread_counts)
+        else:
+            self.t = threading.Timer(60*float(self.settings["updateinterval"]), self.cb_update_unread_counts)
+            
+        self.t.start()
+            
+    def send_notification(self, website, nickname, optype, account_idx, msgs):
+        import pynotify
+
+        #print self.accountsWin.isVisible()
+
+        #diplay_status =self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, MCE_DEVICE_DISPSTATUS_GET, wait_reply=True, use_system_bus=True)  
+        #scr_lock_status =self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, MCE_DEVICE_SCRLOCK_GET, wait_reply=True, use_system_bus=True)  
+
+        if optype == Utils.TIMELINE_TYPE_MENTIONS:
+            keystr = "条@我的新消息"
+        else:
+            keystr = "条新评论"
+            
+        if website==1:
+            iconstr = "maebo_sina"
+        else:
+            iconstr = "maebo_qq"  
+
+        if self.unread_notifiers[str(account_idx)+":t"+str(optype)] != 0 \
+            and self.unread_notifiers[str(account_idx)+":t"+str(optype)] == msgs:
+            return
+
+        print "notify [" +str(account_idx)+":t"+str(optype)+"], msgs", msgs   
+        self.unread_notifiers[str(account_idx)+":t"+str(optype)] = msgs      
+        self.osso_rpc.rpc_run(NOTIFY_DBUS_NAME, NOTIFY_DBUS_PATH, \
+            NOTIFY_DBUS_IFACE, NOTIFY_SHARE_SIG_SHOW, 
+            rpc_args=(str(account_idx), str(optype), str(nickname), \
+            nickname+"﻿收到"+str(msgs)+keystr, iconstr), wait_reply=False, use_system_bus=False)
+            
+        if self.settings["notify_led"] == "True":
+            self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, ENABLE_LED, rpc_args=(LED_PATTERN,"",""),use_system_bus=True)            
+        if self.settings["notify_vibra"] == "True":
+            self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, VIBRATE, rpc_args=(VIBRATE_PATTERN,"",""),use_system_bus=True)            
+        if self.settings["notify_sound"] == "True":
+            bus = dbus.SessionBus()            
+            proxy = bus.get_object('com.nokia.HildonSVNotificationDaemon', '/com/nokia/HildonSVNotificationDaemon')            
+            interface = dbus.Interface(proxy,dbus_interface='com.nokia.HildonSVNotificationDaemon')            
+            #interface.PlayEvent({'time': 0, 'category': 'email-message'}, "maebo")
+            interface.PlayEvent({'time': 0, 'category': 'sms-message'}, "maebo")
+            
+  
+    def rotate_image(self, dst_file):
+        img = Image.open(dst_file)        
+        try:
+            print(img._getexif().items())
+            #exif=dict((ExifTags.TAGS[k], v) for k, v in img._getexif().items())
+            exifinfo = img._getexif()
+            exif={}
+            exif['Orientation'] = 1
+            
+            if exifinfo != None:
+                for tag, value in exifinfo.items():
+                    decoded = TAGS.get(tag, tag)
+                    if decoded == 'Orientation':
+                        exif[decoded] = value
+    
+            print exif['Orientation']
+            if exif['Orientation'] == 1:
+                return
+                
+            if exif['Orientation'] == 6:
+                img=img.rotate(270, expand=True)
+            elif exif[orientation] == 6 : 
+                image=image.rotate(270, expand=True)
+            elif exif[orientation] == 8 : 
+                image=image.rotate(90, expand=True)
+            elif exif[orientation] == 3 : 
+                image=image.rotate(180, expand=True)
+    
+            img.save(dst_file, "JPEG")
+        except:
+            pass
+            
+        
     def send_via_service(self, interface, method, args, user_data):
 
         print "send_via_service"
         
         if method == SH_SHARE_SIG:
+           # args[0] = filepath, args[1] = Title, args[2] = Discirption, args[3] = nickname
            # print args[0], args[1],args[2], args[3]
 
             dst_file = args[0] + ".jpg"
             shutil.copyfile(args[0], dst_file)  
-            
+            self.rotate_image(dst_file)
+
+            if os.path.getsize(dst_file) >= 700*1000:
+                self.show_notice("﻿图片超大，请调整大小", 0)
+                return
+
             param={}
             param["status"]=""
             if args[1] != "":
@@ -160,8 +377,44 @@ class MaoboApp(QApplication):
                     sender.initialize(account["id"],param)
         
                     self.show_notice("微博发送中...")
-                    sender.start()                        
-        else:
+                    sender.start()    
+        elif method == "top_application":
+            print "top_application"
+            self.accountsWin.activateWindow()
+            self.accountsWin.show()
+        elif method == "open_mentions":
+            #args[0] = account_idx, args[1] = optype
+            print "open_mentions", args[0], args[1]
+
+            conn_status = self.osso_rpc.rpc_run("com.nokia.icd", "/com/nokia/icd", \
+                "com.nokia.icd", "get_state", wait_reply=True, use_system_bus=True)
+            print "conn_status",conn_status
+
+            if conn_status == 0 and self.settings["auto_conn"] != "---None---":
+                print "try to auto-connect internet - status: %s" % (self.sm_auto_update_unread)
+
+                #dbus-send --type=method_call --system --dest=com.nokia.icd /com/nokia/icd com.nokia.icd.connect string:"d8a643a0-a0a6-4db2-9b87-759e6fd0f969" uint32:0
+                ret=self.osso_rpc.rpc_run("com.nokia.icd", "/com/nokia/icd", "com.nokia.icd", "connect", \
+                    rpc_args=(self.settings["auto_conn"], 0), wait_reply=True, use_system_bus=True)
+            
+            self.accountsWin.activateWindow()
+            self.accountsWin.show()
+
+            #print self.unread_notifiers[args[0]+":t"+args[1]].isVisible()
+            self.unread_notifiers[args[0]+":t"+args[1]] = 0
+
+            self.change_current_timeline_view(int(args[0]), int(args[1]))
+            
+        elif method == "clear_unread":
+            #args[0] = account_idx, args[1] = optype
+            print "clear_unread", args[0], args[1], "clean unread counts"
+            
+            self.unread_notifiers[args[0]+":t"+args[1]] = 0
+        
+            # clean unread counts
+            #self.unread_updaters[str(self.accounts[int(args[0])]["id"])].initialize(self.accounts[int(args[0])]["id"], Utils.TIMELINE_TYPE_DEUNREAD, optype=int(args[1])) 
+            #self.unread_updaters[str(self.accounts[int(args[0])]["id"])].start()
+        else:    
             print "Not support method: " + method
             return
             
@@ -195,9 +448,9 @@ class MaoboApp(QApplication):
         orientation=None
 
         #rpc = osso.Rpc(osso_c)
-        orientation =self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, MCE_DEVICE_ORIENTATION_GET,
-            wait_reply=True, use_system_bus=True)       
-            
+        orientation =self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, MCE_DEVICE_ORIENTATION_GET, wait_reply=True, use_system_bus=True)       
+        print orientation
+        
         if orientation == 'landscape': 
             return 0
         elif orientation == "landscape (inverted)": 
@@ -226,7 +479,7 @@ class MaoboApp(QApplication):
             Utils.g_orietation = 1
             self.timelineWin.switch(1)
             self.newStatusWin.switch(1)
-            print "portrait"
+            #print "portrait"
         elif orientation ==  "portrait (inverted)":
             Utils.g_orietation = 3
             self.timelineWin.switch(3)
@@ -245,10 +498,11 @@ class MaoboApp(QApplication):
         self.defaultWin.setMenuBar(self.menuBar)
 
     def create_accounts_window(self):
-        self.accountsWin=AccountsWindow(None,self.accounts,self.websites)
+        self.accountsWin=AccountsWindow(None,self.accounts,self.websites, mode=self.settings["background"])
         self.connect(self.accountsWin.accountsListView,
                      SIGNAL("clicked(const QModelIndex&)"),
                      self.account_clicked)
+                     
         accountsManaAction = self.create_action(unicode("账户管理"), self.show_accounts_mana_form)
         settingAction=self.create_action(unicode("微博设置"), self.show_setting_form)
         delcacheAction=self.create_action(unicode("清理缓存"), self.delete_cache)
@@ -288,12 +542,15 @@ class MaoboApp(QApplication):
             self.show_notice(we)
             
         self.logger.debug(self.currentAccount["username"]+" choosed.")
-        #self.menusWin.setWindowTitle(self.currentAccount["nickname"])
-        #self.menusWin.show()
         self.timelineType=Utils.TIMELINE_TYPE_FRIENDS
         self.timelineWin.currentMenuIdx = 0
         model = self.timelineWin.menuListView.model()
         self.timelineWin.menuListView.setCurrentIndex(model.index(0, 0))
+
+
+        self.unread_updaters[str(self.currentAccount["id"])].initialize(self.currentAccount["id"],Utils.TIMELINE_TYPE_UNREAD)        
+        self.unread_updaters[str(self.currentAccount["id"])].start()
+
         self.show_timeline_window()
 
     def create_menus_window(self):
@@ -317,14 +574,17 @@ class MaoboApp(QApplication):
             self.show_new_status_window()
         else:
             self.timelineWin.currentMenuIdx = index.row()
-            self.timelineType=value
+            self.timelineType=value      
             self.show_timeline_window()
-
+            
+    def on_timeline_window_closed(self):
+        self.currentAccount = None
 
     def create_timeline_window(self):
         self.timelineWin=TimelineWindow(self.accountsWin, orientation=Utils.g_orietation)
         self.connect(self.timelineWin.statusesListView,SIGNAL("clicked(const QModelIndex&)"),self.status_clicked)
         self.connect(self.timelineWin.menuListView,SIGNAL("clicked(const QModelIndex&)"),self.menu_clicked)
+        self.connect(self.timelineWin,SIGNAL("timelinwin_close()"),self.on_timeline_window_closed)
         
         self.updateOpe=Utils.build_an_operation(Utils.STATUS_OPE_TYPE_UPDATE,
                                           "Click to update",
@@ -349,7 +609,7 @@ class MaoboApp(QApplication):
 
     def show_timeline_window(self):
         self.logger.debug("show_timeline_window")
-#        self.currentAccountId=self.accounts[0]["id"]
+        
         self.updateOpe["lastUpdated"]=Utils.get_last_update_time_by_timeline_type(self.currentAccount,self.timelineType)
         if self.timelineType==Utils.TIMELINE_TYPE_FAVOUR:
             self.logger.debug("load favorited statuses from db")
@@ -387,6 +647,16 @@ class MaoboApp(QApplication):
                 self.update_timeline()
             elif type==Utils.STATUS_OPE_TYPE_MORE:
                 self.load_more_timeline()
+                
+            # clean unread counts
+            if self.timelineWin.menuModel.need_clear_unread_count(self.timelineType):
+                self.unread_updaters[str(self.currentAccount["id"])].initialize(self.currentAccount["id"], Utils.TIMELINE_TYPE_DEUNREAD, optype=self.timelineType) 
+                self.unread_updaters[str(self.currentAccount["id"])].start()
+
+            self.timelineWin.menuModel.clear_unread_count(self.timelineType) 
+            self.timelineWin.menuListView.hide()
+            self.timelineWin.menuListView.show()
+            
         except KeyError,ae:
             if status["if_gap"]==1:
                 self.logger.debug("it is a gap")
@@ -476,7 +746,7 @@ class MaoboApp(QApplication):
             return
         self.statusWin.commentsOpe["name"]="updating, please wait"
         self.statusWin.commentsOpe["lastUpdated"]=None
-        self.statusWin.load_status()
+        self.statusWin.load_status(self.settings["down_orgpic"])
         self.statusWin.setWindowTitle(self.currentAccount["nickname"])
         self.statusWin.currentOpe=self.statusWin.commentsOpe
         
@@ -532,8 +802,8 @@ class MaoboApp(QApplication):
             
             self.updater.initialize(self.currentAccount["id"],self.timelineType,sinceId,None,self.settings["updatenum"],self.settings["downpic"])
         else:
-    	    pageflag=0
-    	    pagetime=0
+            pageflag=0
+            pagetime=0
     	    
     	    # rem for binkcn 0926
     	    #if len(self.statuses)>1:
@@ -565,6 +835,74 @@ class MaoboApp(QApplication):
                 
         return 0
         
+    def is_app_background_mode(self):
+    
+        if self.activeWindow() == None:
+            return True
+            
+        diplay_status =self.osso_rpc.rpc_run(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, \
+            MCE_DEVICE_DISPSTATUS_GET, wait_reply=True, use_system_bus=True) 
+            
+        if diplay_status == "off":  
+            return True
+
+        return False
+           
+
+        
+    def get_unead_finished(self,accountid):
+        #print "get_unead_finished, id", accountid
+
+        unread_counts = self.unread_updaters[str(accountid)].unread_counts
+        
+        #for n in unread_counts:
+        #    print n
+        #    print unread_counts[n]
+
+        account_idx = 0
+        for acc in self.accounts:
+            if acc["id"] == accountid:
+                break
+            account_idx = account_idx + 1
+
+        send_all = True
+        if send_all == True or self.currentAccount == None or accountid != self.currentAccount["id"] \
+            or self.is_app_background_mode():    
+            account_idx = 0
+            for acc in self.accounts:
+                if acc["id"] == accountid:
+                    break
+                account_idx = account_idx + 1
+
+            if acc["auto_update"] == "True" and unread_counts["mentions"] > 0:
+                self.send_notification(acc["website"], acc["nickname"], \
+                    Utils.TIMELINE_TYPE_MENTIONS, account_idx, unread_counts["mentions"])
+        
+            if acc["auto_update"] == "True" and unread_counts["comments"] > 0:
+                self.send_notification(acc["website"], acc["nickname"], \
+                    Utils.TIMELINE_TYPE_COMMENTS, account_idx, unread_counts["comments"])
+
+        if self.currentAccount != None and accountid == self.currentAccount["id"]:
+            self.timelineWin.menuModel.set_unread_counts(unread_counts)
+            self.timelineWin.menuListView.hide()
+            self.timelineWin.menuListView.show()
+
+        for acc in self.unread_updaters:
+            if self.unread_updaters[acc].isRunning() == True and acc != str(accountid):
+                #print "self.unread_updaters, ", acc, "is running"
+                return
+           #elif self.unread_updaters[acc].isRunning() != True:
+                #print "self.unread_updaters, ", acc, "is not running"
+                
+        if self.sm_auto_update_unread == _SM_AUTO_CONNECTED and self.is_app_background_mode():  
+            print "try to auto-disconnect internet"
+            bus=dbus.SystemBus()
+            icd2_obj = bus.get_object('com.nokia.icd2', '/com/nokia/icd2') 
+            icd2_iface = dbus.Interface(icd2_obj, 'com.nokia.icd2')
+            icd2_iface.disconnect_req(dbus.UInt32(32768))
+            self.sm_auto_update_unread = _SM_AUTO_IDLE
+
+        
     def update_comments_finished(self,completed):
         #print "update_comments_finished"
         self.logger.debug("update_comments_finished")
@@ -583,20 +921,18 @@ class MaoboApp(QApplication):
             self.statusWin.comment_statuses=newStatuses
 
             self.statusWin.setWindowTitle(self.currentAccount["nickname"])
-            self.statusWin.load_status()
+            self.statusWin.load_status(self.settings["down_orgpic"])
         else:
-            self.statusWin.load_status()
- 
-                        
+            self.statusWin.load_status(self.settings["down_orgpic"])
+                       
         self.statusWin.currentOpe=None
         
-    
     def update_finished(self,completed):
         #print "update_finished"
         self.logger.debug("update_finished")
         self.set_windows_status(0)
         newStatuses=self.updater.timeline
-        
+
         # parse remove exist. for tencent.
         if self.currentAccount["website"] == 2:
             tmpStatuses = []
@@ -606,6 +942,11 @@ class MaoboApp(QApplication):
             for newStatus in tmpStatuses:
                 if self.checkExits(newStatus)==1:
                     newStatuses.remove(newStatus)
+                    
+        if self.unread_updaters[str(self.currentAccount["id"])].isRunning() != True:
+            print "update_finished: update unread counts"
+            self.unread_updaters[str(self.currentAccount["id"])].initialize(self.currentAccount["id"],Utils.TIMELINE_TYPE_UNREAD)        
+            self.unread_updaters[str(self.currentAccount["id"])].start()
 
         if self.currentOpe==self.updateOpe:
             #update the lastUpdateTime for the updateOpe and account
@@ -677,6 +1018,9 @@ class MaoboApp(QApplication):
         else:
             self.timelineWin.load_statuses(self.statuses)
             
+        if self.activeWindow() != self.timelineWin and self.activeWindow() != self.accountsWin:
+            self.activeWindow().close()
+        
         self.currentOpe=None
         
     def getMinCreatedAt(self, newstatuses, website):
@@ -765,7 +1109,7 @@ class MaoboApp(QApplication):
         self.statusWin.comment_statuses=[]
             
         self.statusWin.show()
-        self.statusWin.load_status()
+        self.statusWin.load_status(self.settings["down_orgpic"])
         self.statusWin.setWindowTitle(self.currentAccount["nickname"])
         #self.statusWin.switch(self.orientation)
         self.statusWin.show()
@@ -916,7 +1260,8 @@ class MaoboApp(QApplication):
             self.controller.save_an_account(account)
 
             self.newAccountForm.hide()
-            self.defaultWin.hide()
+            if self.defaultWin != None:
+                self.defaultWin.hide()
             self.accountsWin.show()
             self.show_accounts_mana_form()
             self.accountsModel=AccountsModel(self.accounts,self.websites)
@@ -968,7 +1313,11 @@ class MaoboApp(QApplication):
             settings = self.controller.save_settings(settings)
             if settings!=None:
                 self.settings=settings
-                self.show_notice("成功保存微博设置")
+                #self.show_notice("成功保存微博设置")
+        if self.accountsWin != None:
+            self.accountsWin.set_running_mode(self.settings["background"])
+
+        self.settingForm.hide()
 
     def show_about_form(self):
         aboutForm=AboutForm(self.accountsWin)
@@ -1019,27 +1368,41 @@ class MaoboApp(QApplication):
             else:
                 target.addAction(action)
                 
-# Define a callback that gets called for orientation changes
-def handler(orientation, stand, face, x, y, z):
-    print orientation, stand, face, x, y, z
-        
-def main():
-    dir=Utils.DATA_FILE_PATH
-    if os.access(dir, os.W_OK) == 0:
-        os.mkdir(dir)
-    dir=Utils.THUMBNAIL_DIR
-    if os.access(dir, os.W_OK) == 0:
-        os.mkdir(dir)
-    dir=Utils.AVATAR_DIR
-    if os.access(dir, os.W_OK) == 0:
-        os.mkdir(dir)
-    # set up logging to file - see previous section for more details
-    logging.basicConfig(level=logging.INFO,
-                        format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
-                        datefmt='%y-%m-%d %H:%M',
-                        filename=Utils.LOG_FILE,
-                        filemode='a')
-    # define a Handler which writes INFO messages or higher to the sys.stderr
+class MaeboService(dbus.service.Object):
+    def __init__(self, app):
+        self.app = app
+        bus_name = dbus.service.BusName(SESSION_DBUS_NAME, bus = dbus.SessionBus())
+        dbus.service.Object.__init__(self, bus_name, SESSION_DBUS_PATH)
+
+    @dbus.service.method(dbus_interface=SESSION_DBUS_IFACE)
+    def show_window(self):
+        self.app.accountsWin.activateWindow()   
+        self.app.accountsWin.show()             
+
+def main():    
+    if dbus.SessionBus().request_name(SESSION_DBUS_NAME) != dbus.bus.REQUEST_NAME_REPLY_PRIMARY_OWNER:
+    #if False:
+        print "application already running"
+        method = dbus.SessionBus().get_object(SESSION_DBUS_NAME, SESSION_DBUS_PATH).get_dbus_method("show_window")
+        method()
+    else:
+	    print str(time.time()) + ": " + "main"
+	    dir=Utils.DATA_FILE_PATH
+	    if os.access(dir, os.W_OK) == 0:
+	        os.mkdir(dir)
+	    dir=Utils.THUMBNAIL_DIR
+	    if os.access(dir, os.W_OK) == 0:
+	        os.mkdir(dir)
+	    dir=Utils.AVATAR_DIR
+	    if os.access(dir, os.W_OK) == 0:
+	        os.mkdir(dir)
+	    # set up logging to file - see previous section for more details
+	    logging.basicConfig(level=logging.INFO,
+	                        format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
+	                        datefmt='%y-%m-%d %H:%M',
+	                        filename=Utils.LOG_FILE,
+	                        filemode='a')
+	    # define a Handler which writes INFO messages or higher to the sys.stderr
 #    console = logging.StreamHandler()
 #    console.setLevel(logging.DEBUG)
 #    # set a format which is simpler for console use
@@ -1048,19 +1411,27 @@ def main():
 #    console.setFormatter(formatter)
 #    # add the handler to the root self.logger
 #    logging.getLogger('').addHandler(console)
-    reload(sys)
-    sys.setdefaultencoding('utf8')
-    QTextCodec.setCodecForCStrings(QTextCodec.codecForName("UTF-8"))
+	    reload(sys)
+	    sys.setdefaultencoding('utf8')
+	    QTextCodec.setCodecForCStrings(QTextCodec.codecForName("UTF-8"))
+	    print str(time.time()) + ": " + "before MaoboApp"
+	    app = MaoboApp(sys.argv)
+	    app.setOrganizationName("SkyBlue Works")
+	    app.setOrganizationDomain("michaelwong.cn")
+	    app.setApplicationName("maebo")
+	    print str(time.time()) + ": " + "before app.exec_"
+
+	    service = MaeboService(app)
+	    
+
+	    app.exec_()
+	    print "after app.exec_"
+
+	    app.t.cancel()
+#def on_notification_closed (notification, user_data, sbc):
+#    notification.close()
+#    print "on_state_reason_notification_closed"
 
-    app = MaoboApp(sys.argv)
-    app.setOrganizationName("SkyBlue Works")
-    app.setOrganizationDomain("michaelwong.cn")
-    app.setApplicationName("maebo")
-    
-#    form = MainWindow()
-#    form.show()
-    app.exec_()
-    
 
 main()
 
